import color from 'color';
import * as React from 'react';
import { StyleSheet, View } from 'react-native';
import { withTheme } from '../../core/theming';
import Icon from '../Icon';
import TouchableRipple from '../TouchableRipple/TouchableRipple';
import Text from '../Typography/Text';

/**
 * A component used to show an action item with an icon and a label in a navigation drawer.
 *
 * <div class="screenshots">
 *   <figure>
 *     <img class="medium" src="screenshots/drawer-item.png" />
 *   </figure>
 * </div>
 *
 * ## Usage
 * ```js
 * import * as React from 'react';
 * import { Drawer } from 'react-native-paper';
 *
 * const MyComponent = () => (
 *    <Drawer.Item
 *      style={{ backgroundColor: '#64ffda' }}
 *      icon="star"
 *      label="First Item"
 *    />
 * );
 *
 * export default MyComponent;
 * ```
 */
const DrawerItem = /*#__PURE__*/React.forwardRef((_ref, ref) => {
  let {
    icon,
    label,
    active,
    theme,
    style,
    onPress,
    numberOfLines,
    right,
    accessibilityRole,
    accessibilityLabel,
    importantForAccessibility,
    accessibilityElementsHidden,
    ...rest
  } = _ref;
  const {
    colors,
    roundness
  } = theme;
  const backgroundColor = active ? color(colors.primary).alpha(0.12).rgb().string() : 'transparent';
  const contentColor = active ? colors.primary : color(colors.text).alpha(0.68).rgb().string();
  const font = theme.fonts.medium;
  const labelMargin = icon ? 32 : 0;
  const numLines = typeof numberOfLines === 'undefined' ? 1 : numberOfLines;
  let labelElement = label;

  if (typeof label === 'string') {
    labelElement = /*#__PURE__*/React.createElement(Text, {
      numberOfLines: numLines,
      selectable: false,
      style: [styles.label, {
        color: contentColor,
        ...font,
        marginLeft: labelMargin
      }]
    }, label);
  }

  return /*#__PURE__*/React.createElement(View, rest, /*#__PURE__*/React.createElement(TouchableRipple, {
    borderless: true,
    delayPressIn: 0,
    onPress: onPress,
    style: [styles.container, {
      backgroundColor,
      borderRadius: roundness
    }, style],
    ref: ref // @ts-expect-error We keep old a11y props for backwards compat with old RN versions
    ,
    accessibilityTraits: active ? ['button', 'selected'] : 'button',
    accessibilityComponentType: "button",
    accessibilityRole: accessibilityRole !== null && accessibilityRole !== void 0 ? accessibilityRole : 'button',
    accessibilityState: {
      selected: active
    },
    accessibilityLabel: accessibilityLabel,
    accessibilityElementsHidden: accessibilityElementsHidden
  }, /*#__PURE__*/React.createElement(View, {
    style: styles.wrapper,
    importantForAccessibility: importantForAccessibility
  }, /*#__PURE__*/React.createElement(View, {
    style: styles.content
  }, icon ? /*#__PURE__*/React.createElement(Icon, {
    source: icon,
    size: 24,
    color: contentColor
  }) : null, labelElement), right === null || right === void 0 ? void 0 : right({
    color: contentColor
  }))));
});
DrawerItem.displayName = 'Drawer.Item';
const styles = StyleSheet.create({
  container: {
    marginHorizontal: 10,
    marginVertical: 4
  },
  wrapper: {
    flexDirection: 'row',
    alignItems: 'center',
    padding: 8
  },
  content: {
    flex: 1,
    flexDirection: 'row',
    alignItems: 'center'
  },
  label: {
    marginRight: 32
  }
});
export default withTheme(DrawerItem);
//# sourceMappingURL=DrawerItem.js.map